### ContentPackage: IMS Content Packaging support for Zope
### Copyright (C) 2002 Scott Wilson (s.wilson@bangor.ac.uk)
###
### This library is free software; you can redistribute it and/or
### modify it under the terms of the GNU Lesser General Public
### License as published by the Free Software Foundation; either
### version 2.1 of the License, or (at your option) any later version.
###
### This library is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
### Lesser General Public License for more details.
### 
### You should have received a copy of the GNU Lesser General Public
### License along with this library; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###
###
### Main module for the Content Package Zope product
###
### Provides capability in Zope to work with
### IMS Content Packages
###
### Scott Wilson 15 Nov 2002
### Fixed some pointer issues with the Export function.
### //released with Content Package v0.3//
###
### Scott Wilson 5 Nov 2002
### Inherited from new SCORM API module
###
### Scott Wilson 31 Oct 2002
### Added export capability
### //released with Content Package v0.2//
###
### Scott Wilson Sept/Oct 2002
### First cut
### //released with Content Package v0.1//
###
### @author Scott Wilson (s.wilson@bangor.ac.uk)
### 
### Acknowledgements
### ================
### The basis for this product was ZipFolder, by Jens Quade.
### Also thanks to Jens for the __write_data method!
###
### To-Do List
### ==========
### Allow for manifest-only "packages"
### Treat a folder with a manifest in it as a nested ContentPackage!
### Make the zip export a bit nicer, and use the built-in
### tempfile capability...
###

from OFS import Folder,Image
from Globals import HTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions
from ZODB.ExportImport import TemporaryFile, ExportImport

import os
import re
import string

import types
import pickle


import StringIO
import zipfile
import CPImporter
import Manifest
from Metadata import *
from SCORMRuntime import *

# check zip file names for:
filenamepattern=re.compile(r'[A-Za-z0-9][\w\_\-\.]*$')

MINZIPFILESIZE=22


manage_addContentPackageForm=HTMLFile('dtml/ContentPackageAdd', globals())


def manage_addContentPackage(self, id, title='',file=None,subfolders=0,REQUEST=None):
    """Add a new CP object with id *id*, uploading file.
    """
    id=str(id)
    id, title = Image.cookId(id, '', file)

    ob=ContentPackage()
    ob.id=id
    
    self._setObject(id, ob)
    ob=self._getOb(id)

    checkPermission=getSecurityManager().checkPermission


    if file is not None:
        if not checkPermission('Add Documents, Images, and Files', ob):
            raise 'Unauthorized', (
                  'You are not authorized to add DTML Documents.'
                  )
        return ob.manage_upload(file,subfolders,REQUEST=REQUEST)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class ContentPackage(Folder.Folder, CPImporter.CPImporter,LomDescribedObject,SCORMRuntime):
    """CP main class"""

    meta_type="ContentPackage"

    __ac_permissions__=(
        (Permissions.view_management_screens,
         ('manage_uploadForm',)),
        ('Change ContentPackages',
         ('manage_upload','manage_uploadForm')))

    
    manage_options=(LomDescribedObject.manage_options+(
                       { "label": "Organisation",
                         "action": "manage_viewOrganisation"},
                       { "label": "Upload",
                         "action": "manage_uploadForm"},
                       { "label": "Download",
                          "action": "manage_download"},
                       { "label": "Manifest",
                         "action": "manage_viewManifest"},
                       {'label':'Contents', 
                        'action':'manage_main',
                        'help':('OFSP','ObjectManager_Contents.stx')}
                       )
                    )

    manage_viewOrganisation=HTMLFile('dtml/organisations',globals())
    manage_uploadForm=HTMLFile('dtml/ContentPackageUpload', globals())
    manage_viewManifest=HTMLFile('dtml/ContentPackageManifest',globals())
    manage_download=HTMLFile('dtml/ContentPackageDownload',globals())
    manage_viewResource=HTMLFile('dtml/resource',globals())
    SCORM_API=HTMLFile('dtml/api',globals())
    SCORM_STATUS=HTMLFile('dtml/status',globals())


    ##
    ## Convenience methods for accessing common Manifest functions
    ##

    def getManifest(self):
        """Return the manifest for the package"""
        return self.Manifest

    def getDefaultOrganisationIdentifier(self):
        """Return the identifier of the default organisation in the Manifest"""
        return self.Manifest.getDefaultOrganisationIdentifier()

    def getOrganisations(self):
        """Get the list of organisations from the manifest"""
        return self.Manifest.getOrganisations()

    def getOrganisation(self,identifier):
        """Get the organisation identified from the manifest"""
        return self.Manifest.getOrganisation(identifier)

    def getResources(self,identifier):
        """Return a dictionary of resources for the identified org in the manifest"""
        return self.Manifest.getResources(identifier)

    def containsSubManifests(self):
        """Returns true if the manifest for this package contains sub-manifests"""
        return self.getManifest().containsSubManifests()

    def getMetadata(self,dictionaryName,fields):
        """Return the metadata dictionary for the package"""
        return self.getManifest().getMetadata(dictionaryName,fields)          

    def getMetadataDictionary(self,dictionaryName,objectType,identifier):
        """Return the metadata dictionary for the package"""
        return self.getManifest().getMetadataDictionary(dictionaryName,objectType,None)  

    ##
    ## Methods for uploading and constructing packages, extend the CPImporter methods
    ##

    def manage_upload(self,file='',subfolders=0,replace=0,REQUEST=None, RESPONSE=None):
        """Accept a file and load it up"""

        if isinstance(file,types.StringType):
            if REQUEST: return MessageDialog(
                title  ='Success!',
                message='But no ZIP file.',
                action ='manage_main')
            return # anyway
        
        # check file sizes
        file.seek(0,2)
        fsize=file.tell()
        file.seek(0,0)

        if fsize == 0:
            if RESPONSE:
                RESPONSE.redirect('manage_main')
            return # anyway

        if replace:
            if subfolders:
                self._remove_objects(metatypes=('DTML Document','Image',
                                                'File','Folder','Manifest'))
            else:
                self._remove_objects(metatypes=('DTML Document','Image',
                                                'File','Manifest'))
        

        self._v_skipped=[]
        self._v_subfolders=subfolders
        self.import_zipfile(file)
        
        if REQUEST:
            if len(self._v_skipped)>0:
                return MessageDialog(
                    title  ='Success!',
                    message=('ZIP file uploaded successfully, '+
                             'some files/folders ignored:<br>'+
                             string.join(self._v_skipped,'<br>')),
                    action ='manage_main')
            else:
                return MessageDialog(
                    title  ='Success!',
                    message='ZIP file uploaded successfully',
                    action ='manage_main')

        if len(self._v_skipped) > 0:
            return self._v_skipped

        return None
            
    def _remove_objects(self,metatypes):
        """Removes old objects from the folder"""
        
        removelist=self.objectIds(metatypes)
        self.manage_delObjects(ids=removelist)

                
    def _get_subfolder(self,path):

        dirlist = []
        rest = ""
        while path != rest:
            rest=path
            (path,dir)=os.path.split(path)
            if dir != "":
                dirlist.append(dir)

        dirlist.reverse()

        folder=self
        path=""

        for dir in dirlist:
            if self.check_filename(path,dir):
                
                if dir in folder.objectIds():
                    if not folder[dir].isPrincipiaFolderish:
                        
                        raise RuntimeError,("Object '%s' already exists "+
                                            "and is no folder: Cannot "+
                                            "create or use folder") % dir
                else:
                    ### IN future need to check to see if the folder
                    ### has a manifest in it - in which case we need to create
                    ### a sub-package :-(
                    folder.manage_addFolder(id=dir)

                folder=folder[dir]

            else:
                return None # no folder, no files in there...

            if path:
                path=path + "/" + dir
            else:
                path=dir
                
        return folder
    
    def _edit_html(self, data):
        # a hook for filters to edit the imported html files
        return data

    def _add_other_file(self,folder,path,basename,sf):
        # Hook for other known file types, default: add file objects
        folder.manage_addFile(id=basename, title='', file=sf)

    def check_filename(self,pathname,basename):
        # return 0 to skip file, raise an exception to annoy...
        match=filenamepattern.match(basename) != None;
        if not match and basename != '':
            self._v_skipped.append(os.path.join(pathname,basename))
        return match


    def check_zip(self,zipfile):
        # take a look and see if we have a manifest file
        hasManifest = 0
        for name in zipfile.namelist():
            if name == 'imsmanifest.xml':
               hasManifest = 1
        return hasManifest


    def add_file(self,path,basename,sf):
        
        if not filenamepattern.match(basename):
            return # skip ugly files
        
        suffix=os.path.splitext(basename)[1]

        folder=self

        if self._v_subfolders:
            folder=self._get_subfolder(path)

        if folder:
            if string.lower(suffix) in ['.htm', '.html']:
                # this is ugly
                sfx=StringIO.StringIO(self._edit_html(sf.getvalue()))
                sf.close()
                folder.manage_addDTMLDocument(id=basename, title='', file=sfx)

	    elif string.lower(suffix)=='.xml' and string.lower(basename)=='imsmanifest.xml':
                sfx=sf.read()
                sf.close()                
                manifest = folder._setObject('Manifest', Manifest.Manifest('Manifest',src=sfx))


            elif string.lower(suffix) in ['.gif', '.jpg', '.jpeg', '.png']:
                folder.manage_addImage(id=basename, title='', file=sf)


            elif len(suffix)==0 and basename[-5:] == "_html":
                folder.manage_addDTMLDocument(id=basename, title='', file=sf)
                
            else:
                self._add_other_file(folder,path,basename,sf)

    # Methods for downloading packages.

    def download(self):
        """Creates a ZipFile containing the Content Package"""     
        print "download"
        cp = zipfile.ZipFile('download.zip','w')

        """Add the manifest"""
        info = zipfile.ZipInfo('imsmanifest.xml',(2002,10,1,0,0,0))
        cp.writestr(info,str(self.getManifest().src))
        msg = "<BR>Wrote imsmanifest.xml"
        """Add all the files and folders"""
        msg = msg + self.__add_files(cp,self,'','/')

        """Close file and reopen for reading"""
        cp.close()
        f = open('download.zip','r+b')
        
        """Delete the old zipfile, if any, and add a new Zope zipfile object"""
        if 'download.zip' in self.objectIds():
           self.manage_delObjects('download.zip')
        self.manage_addFile(id='download.zip',title='Zipfile',file=f.read())

        """Close the file"""
        f.close()

        return "<p>Completed" + msg +"</P>" + '<p><a href="download.zip">click here to download</a></p>'


    def __add_files(self,cp,folder,msg,path):
        """Add objects from package to the zipfile"""
        print "__add_files("+str(folder.getId())+","+path+")"
        for object in folder.objectValues():
            msg=msg+"<br>[OBJECT:"+str(object.getId())+"]"
            if object.meta_type == 'Folder':
               subpath = path + str(object.getId())+'/'
               print "::New path:"+subpath
               self.__add_files(cp,object,msg,subpath)
            else:
               if object.meta_type <> 'Manifest' and object.getId() <> 'download.zip':
                  self.exportObject(cp,object,path)
	       else:
		  print "::[Skipping "+str(object.getId())+"]"
        return msg

    def exportObject(self,zip,object,path):
        """Exports an object to a file and adds it to the zipfile.
           TO DO: Use a TEMPFILEE instead of "download.tmp"
        """     
        id = str(object.getId())
        print "exportObject("+path+id+")"       
        f = open('download.tmp','w+b')
        if object.meta_type == "DTML Document" or object.meta_type == "DTML Method":
	   ## Changed from __str__ to document_src ##
           f.write(object.document_src())
        else:
           self.__write_data(object,f)
        f.close()
        zip.write('download.tmp',path+id)
        return "::Added(exportObject): "+str(object.getId())+", " + str(object.title)
	

    def __write_data(self,object,file):
        """Write data in @object to @file
        """
        data=object.data
        if type(data) is type(''):
           file.write(data)
           return

        while data is not None:
           file.write(data.data)
           data=data.next

   
