### ContentPackage: IMS Content Packaging support for Zope
### Copyright (C) 2002 Scott Wilson (s.wilson@bangor.ac.uk)
###
### This library is free software; you can redistribute it and/or
### modify it under the terms of the GNU Lesser General Public
### License as published by the Free Software Foundation; either
### version 2.1 of the License, or (at your option) any later version.
###
### This library is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
### Lesser General Public License for more details.
### 
### You should have received a copy of the GNU Lesser General Public
### License along with this library; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###
###
### Manifest module for Content Package Zope Product
###
### Provides a Manifest class for manipulating package
### meta-information including metadata, organizations 
### and resources. Depends upon the Metadata module.
###
### Scott Wilson 31 Oct 2002
### Minor readability changes to the code
### //Released with Content Package v0.2//
###
### Scott Wilson Sept/Oct 2002
### First cut.
### //Released with Content Package v0.1//
###
### @author Scott Wilson (s.wilson@bangor.ac.uk)
###

from OFS import SimpleItem
from Globals import HTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions
from xml.dom.minidom import *
from xml.dom import *

from Metadata import *
from CPXmlHelper import *

import os
import re
import string
import tempfile
import types

import StringIO

manage_addManifestForm=HTMLFile('dtml/ManifestAdd', globals())


def manage_addManifest(self,id,src,REQUEST=None):
    """Add a new Manifest object with id *id*.
    """
    id=str(id)

    ob=Manifest()
    ob.id=id
    ob.src=src

    self._setObject(id, ob)
    ob=self._getOb(id)

    checkPermission=getSecurityManager().checkPermission

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class Manifest(SimpleItem.SimpleItem, CPXmlHelper, LomDescribedObject):
    """IMS Content Packaging Manifest class"""

    meta_type="Manifest"
    
    manage_options=(LomDescribedObject.manage_options+
                    ( 
                     { "label": "Organisations",
                       "action": "manage_viewOrganisations"},
                     { "label": "Resources",
                       "action": "manage_viewResources"},
                     { "label": "XML",
                       "action": "manage_viewSource"},
                     )
                    )

    manage_viewOrganisations=HTMLFile('dtml/organisations',globals())
    manage_viewSource=HTMLFile('dtml/ManifestXML', globals())
    manage_viewResources=HTMLFile('dtml/ManifestResources', globals())

    def __init__(self,id,src):
        self.id = id
        self.src = src

    def __getDocument(self):
        """Return a DOM Document parsed from the XML source of the manifest
        """
        d = parseString(self.src)
        return  d

    def getURL(self,d,identifier):
        """Return a URL for the resource specified by *identifier*.
        """
        theUrl = ''
        for element in d.getElementsByTagName('resource'):
            if element.getAttribute('identifier')==identifier:
               theUrl = element.getAttribute('href')
        return theUrl

    def containsSubManifests(self):
        """Return True if this manifest has any sub-manifests in it
        """
        d = self.__getDocument()
        l = len(d.getElementsByTagName('manifest'))
        if l > 1:
            return 1

    def getDefaultManifestIdentifier(self):
        """Return a string containing the identifier for the first manifest in the manifest
        """
        d = self.__getDocument()
        identifier = None
        manifests = d.getElementsByTagName('manifest')
        if manifests <> None and len(manifests)>0:
           identifier = manifests[0].getAttribute('identifier')
        return identifier      

    def __getChildResourceItems(self,d,node,level):
        """Return a mapping item containing information about resource items in an organisation.
           Called from getResources, this recursively looks through the 'item' nodes in the organisation
           returning mappings for URL, title and nesting level for each.
        """
        rc = []
        for item in node.childNodes:
            if item.nodeName ==  'item':
               for element in item.childNodes:
                   if element.nodeName == 'title':
                      sTitle = str(self.getText(element.childNodes))
               sUrl = str(self.getURL(d,item.getAttribute('identifierref')))
               resource = [{'title':sTitle, 'URL':sUrl, 'level':level}]
               rc = rc + resource + self.__getChildResourceItems(d,item,level+1)
        return rc

    def getAllResources(self):
        """Return all resources listed in the manifest
        """
        resources = []
        d = self.__getDocument()
        resourcelist = d.getElementsByTagName('resource')
        for resourceitem in resourcelist:
            resources = resources + [{'type':str(resourceitem.getAttribute('type')),'identifier': str(resourceitem.getAttribute('identifier')),'href': str(resourceitem.getAttribute('href'))}]
        return resources

    def getResources(self,identifier):
        """Return a list of resources for an organisation located by its identifier.
           The list takes the form of a list of dictionaries, each with items for the
           keys title, URL and level. Level is the nesting level of the item in the
           organisation.
        """
        d = self.__getDocument()
        resources = []
        organisations = d.getElementsByTagName('organization')
        for organisation in organisations:
           if organisation.getAttribute('identifier')==identifier:
                 resources = resources + self.__getChildResourceItems(d, organisation,1)
        return resources

    def getDefaultOrganisationIdentifier(self):
        """Return a string containing the identifier of the default organisation
           for the manifest. Returns an empty string if no default exists.
           If the manifest contains submanifests, returns the identifier of the default
           organisation for the first manifest (or an empty string) but does not
           look inside any sub-manifests.
        """
        d = self.__getDocument()
        identifier = ''
        organizations = d.getElementsByTagName('organizations')
        if organizations <> None and len(organizations)>0:
           identifier = organizations[0].getAttribute('default')
        return identifier

    def getOrganisation(self,identifier):
        """Return a dictionary containing the title and identifier of any organisations matching
           the supplied *identifier*.
        """
        d = self.__getDocument()
        org = []
        if identifier == None:
           identifier = self.getDefaultOrganisationIdentifier()
        organisations = d.getElementsByTagName('organization')
        for organisation in organisations:
           if organisation.getAttribute('identifier')==identifier:
              for element in organisation.childNodes:
                  if element.nodeName == 'title':
                     orgTitle = self.getText(element.childNodes)
              org = org + [{'title':str(orgTitle), 'identifier':str(organisation.getAttribute('identifier'))}]
        return org

    def getOrganisations(self):
        """Return a list of dictionaries, each containing the title and identifier for an organisation in the manifest
        """
        d = self.__getDocument()
        orgList = []
        organisations = d.getElementsByTagName('organization')
        for organisation in organisations:
            for element in organisation.childNodes:
                if element.nodeName == 'title':
                   orgTitle = self.getText(element.childNodes)
                   if orgTitle == '': orgTitle = 'untitled'
            org = [{'title':str(orgTitle), 'identifier':str(organisation.getAttribute('identifier'))}]
            orgList = orgList + org
        return orgList

    
    # Metadata - public interfaces
    

    def getMetadata(self,dictionaryName,fields):
        """Return a set of specific metadata fields, in a dictionary, for this object only
        """
        return self.__getMetadataDictionary(dictionaryName,'manifest',None,fields,1)

    def getMetadataDictionary(self,dictionaryName,objectType,identifier):
        """Return a specific dictionary of metadata (keywords and values) for the object identified using the supplied
           type and identifier.
        """
        return self.__getMetadataDictionary(dictionaryName,objectType,identifier,None,0)
 
   
    # Metadata - private implementation
    

    def __getMetadataDictionary(self,dictionaryName,objectType,identifier,fields,fixedFields):
        """Return a specific dictionary of metadata (keywords and values) for the object identified             
           using the supplied type and identifier. Passes information about required fields and the 
           type of mapping required to LomMetadata.
        """
        md = None
        if objectType == 'manifest':
           md = self.__getMetadataForManifest(identifier)
        elif objectType == 'organisation':
           md = self.__getMetadataForOrganisation(identifier)
        else:
           md = self.__getMetadataForObject(objectType,identifier)
        if md <> None:
           lom = LomMetadata(md)
           if fixedFields:
              return lom.getMapping(dictionaryName,fields)
           else:
              if fields:
                 return lom.getDictionaryFields(dictionaryName,fields)
              else:
                 return lom.getDictionary(dictionaryName)
        else:
           return None

    
    # The getMetadataFor...() methods below all support the getMetadataDictionary method, and provide
    # a reference to the <metadata> node for the object they interrogate
    

    def __getMetadataForManifest(self, identifier):
        if identifier == None:
           identifier = self.getDefaultManifestIdentifier()
        return self.__getMetadataForObject('manifest',identifier)

    def __getMetadataForOrganisation(self,identifier):
        """Return the metadata node for the organisation matching the identifier parameter
        """
        md = None
        d = self.__getDocument()
        if identifier == None:
           identifier = self.getDefaultOrganisationIdentifier()
        return self.__getMetadataForObject('organization',identifier)

    def __getMetadataForObject(self,obType,identifier):
        md = None
        d = self.__getDocument()
        objects = d.getElementsByTagName(obType)
        for object in objects:
           if (identifier == None or identifier == '') or (object.getAttribute('identifier')==identifier):
              if object.hasChildNodes():
                for element in object.childNodes:
                  if self.getLocalName(element.nodeName) == 'metadata':
                     md =  element
        return md 
       