### ContentPackage: IMS Content Packaging support for Zope
### Copyright (C) 2002 Scott Wilson (s.wilson@bangor.ac.uk)
###
### This library is free software; you can redistribute it and/or
### modify it under the terms of the GNU Lesser General Public
### License as published by the Free Software Foundation; either
### version 2.1 of the License, or (at your option) any later version.
###
### This library is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
### Lesser General Public License for more details.
### 
### You should have received a copy of the GNU Lesser General Public
### License along with this library; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###
###
### Metadata module for the Content Package Zope product
###
### Provides support for objects described using IEEE
### Learning Object Metadata (LOM) and IMS Learning
### Resource Metadata
###
### Scott Wilson 5 Nov 2002
### Allowed functions to return arrays as well as strings for 
### metadata values; useful for multiple Contributes amongst other
### things
###
### Scott Wilson 1 Nov 2002 
### Included support for complex types such as "Contribute", plus
### all the other main metadata sections, and removed old code.
###
### Scott Wilson 31 Oct 2002
### Included other sections
### //released with Content Package v0.2//
###
### Scott Wilson Sept/Oct 2002
### First cut with General section only
### //released with Content Package v0.1//
###
### @author Scott Wilson (s.wilson@bangor.ac.uk)
###

from Globals import HTMLFile, MessageDialog
from xml.dom.minidom import *
from xml.dom import *

from CPXmlHelper import *

import os
import re
import string
import tempfile
import types

class LomDescribedObject:
  """Mixin class providing basic metadata handling functions for LOM.
     Can be inherited by any class which has a metadata description 
     associated with it, example a Content Package, Manifest, Resource, or
     item.
  """
  manage_options=(
                    ( 
                     { "label": "Metadata",
                       "action": "manage_viewMetadata"},
                     )
                 )
  manage_viewMetadata=HTMLFile('dtml/metadata',globals())


class LomMetadata(CPXmlHelper):
  """Provides the LOM management functions and interfaces for reading metadata
     values for a LomDescribedObject.
  """
  def __init__(self,element):
      self.lom = None
      
      if element.hasChildNodes():
        for item in element.childNodes:
          if self.getLocalName(item.nodeName) == 'lom' or self.getLocalName(item.nodeName) == 'record':
             self.lom = item
      
  def __getDictionary(self,fieldlist,fixedFieldNames,section):
      """Return a simple dictionary of metadata fields specified in 
         *fieldlist* from the section identified using *section*
      """
      gd = None    
      if self.lom <> None:
          section = self.__getSection(section)
          if section <> None:
                  if fieldlist == None:
                      # Default field lists
                      fieldlist = self.__getDefaultFields(section)
                  if fixedFieldNames == 1:
                      gd={}
                      for field in fieldlist:
                          if self.__getItem(field,section)<>'':
                              gd[str(field)]=str(self.__getItem(field,section))
                  else:
                      gd=[]
                      for field in fieldlist:
                          gd=gd+[{'fieldname':field,'value':self.__getItem(field,section)}]            
      return gd   
              
  def __getDefaultFields(self,section):
      """Return the set of default fields for section *section*
      """
      fields = None
      if section <> None:
          if self.getLocalName(section.nodeName) == 'general':
             fields = ['title','description','language','coverage','structure','aggregationlevel']
          elif self.getLocalName(section.nodeName) == 'lifecycle':
              fields = ['version','status','contribute']
          elif self.getLocalName(section.nodeName) == 'metametadata':
              fields = ['catalogentry','contribute','metadatascheme','language']
          elif self.getLocalName(section.nodeName) == 'technical':
              fields = ['format','size','location','requirement','installationremarks','otherplatformrequirements','duration']
          elif self.getLocalName(section.nodeName) == 'educational':
              fields = ['interactivitytype','learningresourcetype','interactivitylevel','semanticdensity','intendeduserrole','learningcontext','typicalagerange','difficulty','typicallearningtime','description','language']                          
          elif self.getLocalName(section.nodeName) == 'rights':
              fields = ['cost','copyrightandotherrestrictions','description']
          elif self.getLocalName(section.nodeName) == 'relation':
              fields = ['kind','resource'] 
          elif self.getLocalName(section.nodeName) == 'annotation':
              fields = ['centity','date','description'] 
          elif self.getLocalName(section.nodeName) == 'classification':
              fields = ['purpose','taxonpath','description','keyword']              
      return fields
          
      
  def __getItem(self,field,section):
	"""Return the value of the field *field* within LOM section *section*
		TO DO:
		CatalogEntry complex type
		Contribute complex type
		Requirement complex type
		Taxonpath complex type
		DateTime types
	"""
	item = ''

	if section <> None:
		if section.hasChildNodes():
	      		if field in ['catalogentry','contribute','requirement','taxonpath']:
				item = self.__getComplexItem(field,section)	     
			else:		
				for node in section.childNodes:
					if self.getLocalName(node.nodeName) == field:
						ns = self.getNS(node.nodeName)
						item = item + self.__getItemValue(node,ns)
	return item
      
  def __getSection(self,sectionName):
      """Return a reference for section given using the string *sectionName*
      """
      section  = None
      if self.lom.hasChildNodes():
        for sec in self.lom.childNodes:
          if self.getLocalName(sec.nodeName) == sectionName:
             section = sec
      return section 
      
  def __getComplexItem(self,field,section):
	"""Return the value of complex field *field* from LOM section *section*"""
        item = ''
	if field == 'contribute':
                item = []
                ns = self.getNS(section.nodeName)
		for node in section.getElementsByTagName(ns + 'contribute'):
                        sitem=''
			for element in node.childNodes:
				if self.getLocalName(element.nodeName)== 'role':
					sitem = sitem + self.__getItemValue(element,ns)
                                if self.getLocalName(element.nodeName)=='centity':
                                        sitem = sitem + ": "
					sitem = sitem + self.__getItemValue(element,ns)
                                if self.getLocalName(element.nodeName)=='date':
                                        sitem = sitem + ", "
					sitem = sitem + self.__getItemValue(element,ns)
                        item = item +[sitem]
	return item
				

					
  def __getItemValue(self,node,ns):
	"""Return a string representing the value of node *node* from the namespace *ns*
	"""
        item = ''
        item = str(self.getTextCN(node))
	for langstring in node.getElementsByTagName(ns +'langstring'):
		item = item + self.getTextCN(langstring)
	for vcard in node.getElementsByTagName(ns +'vcard'):
		item = item + self.__vCard(self.getTextCN(vcard),'FN')
		item = item + self.__vCard(self.getTextCN(vcard),'ORG')
	for temporal in node.getElementsByTagName(ns +'datetime'):
		item = item + '(' + self.getTextCN(temporal) + ')'
	return item

  def __vCard(self,vcard,item):
	"""Parses vCard and returns item specified"""
        result = ''
        pattern = re.compile(item+':(.*?)\s[A-Z]{2,}')
	result = pattern.search(vcard)
        if result == None: 
		result = '' 
	else: 
		result=result.group(1)
	return result


  ##
  ## Main dictionary access methods
  ## 

  def getDictionary(self,section):
      """Return a simple dictionary for the metadata section given. For example:
         {'fieldname':'title', 'value':'mytitle'}
         This is useful for building dynamic lists of metadata
      """
      gd = []
      if self.lom <> None:
         gd = self.__getDictionary(None,0,section)
         return gd

  def getDictionaryFields(self,section,fieldlist):
      """Return a dictionary of the specified fields for the specified section
      """
      gd = []
      if self.lom <> None:
         gd = self.__getDictionary(fieldlist,0,section)
         return gd

  def getMapping(self,section,fieldlist):
      """Returns a dictionary of key:value pairs, for example {title:mytitle,description:mydescription}
         This is useful for some DTML applications where you want hard-coded field names for simpler UI
         code."""
      if self.lom <> None:
         return self.__getDictionary(fieldlist,1,section)
     
      